from nib import Processor, after

@after
class IndexPages(Processor):
    def process(self, documents, resources):
        indexes = {}

        for document in documents:
            if 'index' in document:
                index = document['index']

                if index not in indexes:
                    indexes[index] = list()

                indexes[index].append(document)

        for document in documents:
            if document.path in indexes:
                document['pages'] = indexes[document.path]

        return documents, resources



import datetime
import re
from nib import Document, Processor, before, document

dateregex = re.compile(r'(?P<year>\d\d\d\d)[-./](?P<month>\d\d)[-./](?P<day>\d\d)')

@before
class BlogDateProcessor(Processor):
    def document(self, document):
        if 'date' in document:
            if document.group is None:
                document.group = 'blog'

        return document

@document('blog')
class BlogDocumentProcessor(Processor):
    def process(self, documents, resources):
        archives = self.options['blog']['archive']
        uris = self.options['blog']['uris']
        templates = self.options['blog']['templates']
        blog_pages = {}

        def blog_page(name, parent=None, child=None, **kwargs):
            path = uris[name].format(**kwargs)

            if path not in blog_pages:
                page = Document(path=path,
                                content='',
                                short='',
                                template=templates[name],
                                pages=[],
                                **kwargs
                                )
                if parent:
                    parent['pages'].append(page)
                blog_pages[path] = page
            else:
                page = blog_pages[path]

            if child:
                page['pages'].append(child)

            return page

        feed_page = blog_page('feed')
        index_page = blog_page('index')
        archive_page = blog_page('archive', title='Archive')
        tags_page = blog_page('tags', title='Tags')

        for document in documents:
            document['template'] = templates['post']

            if type(document['date']) == datetime.date:
                date = document['date']
                kwargs = {
                    'year': date.year,
                    'month': date.month,
                    'day': date.day,
                }

                if archives['yearly']:
                    blog_page('yearly', parent=archive_page, child=document,
                              title=date.strftime('%Y'), **kwargs)
                if archives['monthly']:
                    blog_page('monthly', parent=archive_page, child=document,
                              title=date.strftime('%B %Y'), **kwargs)
                if archives['daily']:
                    blog_page('daily', parent=archive_page, child=document,
                              title=date.strftime('%B %d, %Y'), **kwargs)

            if 'tags' in document:
                tags = [token.strip() for token in document['tags'].split(',')]
                document['tags'] = {}
                for tag in tags:
                    tag_page = blog_page('tag', parent=tags_page, child=document,
                                         title=tag, tag=tag)
                    document['tags'][tag] = tag_page

            feed_page['pages'].append(document)
            index_page['pages'].append(document)

        documents.extend(blog_pages.values())

        return documents, resources



import re
import yaml

document_marker = re.compile(r'^---\s*$', re.M)

try:
    from yaml import CSafeLoader as SafeLoader
    from yaml import CSafeDumper as SafeDumper
except ImportError:
    from yaml import SafeLoader, SafeDumper

def load(filename, supplement=False):
    with open(filename) as f:
        content = f.read()

    documents = document_marker.split(content, 2)
    data = yaml.load(documents.pop(0), Loader=SafeLoader)

    if supplement:
        return data, documents
    else:
        return data

def save(filename, data, supplement=None):
    output = yaml.dump(dict(data), Dumper=SafeDumper, indent=4,
                       default_flow_style=False)
    with open(filename, 'w') as f:
        f.write(output)

        if supplement:
            f.write('---\n')
            f.write(supplement)




from os import path

from nib import Resource, Processor, after

apache_redirects = b"""
RewriteCond %{DOCUMENT_ROOT}/$1/index.html -f
RewriteRule ^(.*)$ /$1/index.html [L]

RewriteCond %{DOCUMENT_ROOT}/$1.html -f
RewriteRule ^(.*)$ /$1.html [L]

RewriteCond %{DOCUMENT_ROOT}/$1/index.html -f
RewriteRule ^(.*)/$ /$1 [R]

RewriteCond %{DOCUMENT_ROOT}/$1.html -f
RewriteRule ^(.*)/$ /$1 [R]
"""

apache_redirects_base = b"""
RewriteEngine on
RewriteBase /
"""

@after
class PrettyURLProcessor(Processor):
    def process(self, documents, resources):
        for document in documents:
            filename = path.basename(document.uri)
            if filename == 'index.html':
                document.uri = path.dirname(document.path)
            elif document.extension == '.html':
                document.uri = document.path

        htaccess = None
        for resource in resources:
            if resource.path == '.htaccess':
                htaccess = resource

        if not htaccess:
            htaccess = Resource(path='.htaccess',
                                content=apache_redirects_base)
            resources.append(htaccess)

        htaccess.content += apache_redirects

        return documents, resources



version = '0.4.1'

from os import path
cwd = path.abspath(path.dirname(__file__))

import nib.yaml as yaml
from nib.config import Config
from nib.document import Document
from nib.resource import Resource
from nib.render import Render, jinja
from nib.processor import Processor
from nib.processor import before, after, document, resource, markup
from nib.build import Build

import nib.plugins



import os
from os import path
from urllib.parse import urljoin

from nib import Document, Resource, Render
from nib.processor import preprocessors, postprocessors,\
    document_processors, resource_processors, markup_processors
import nib.plugins

class Build(object):
    def __init__(self, options):
        self.document_path = options['document_path']
        self.resource_path = options['resource_path']
        self.output_path = options['output_path']
        self.options = options

        nib.plugins.load(options)

    def load(self):
        documents = []

        for root, dirs, files in os.walk(self.document_path):
            for filename in files:
                filepath = path.join(root, filename)

                try:
                    print('Reading {}'.format(filepath))
                    document = Document.from_file(filepath, self.options)
                    documents.append(document)

                except Exception as e:
                    print('Error while reading {}: {}'.format(filepath, e))

        resources = []

        for root, dirs, files in os.walk(self.resource_path):
            for filename in files:
                filepath = path.join(root, filename)

                try:
                    print('Reading {}'.format(filepath))
                    resource = Resource.from_file(filepath, self.options)
                    resources.append(resource)

                except Exception as e:
                    print('Error while reading {}: {}'.format(filepath, e))

        return documents, resources

    def process_documents(self, documents, resources):
        # break documents into groups by type
        documents_by_group = {}
        for document in documents:
            group = document.group
            if group not in documents_by_group:
                documents_by_group[group] = []
            documents_by_group[group].append(document)

        # process documents by group
        documents = []
        for group in documents_by_group:
            group_documents = documents_by_group[group]

            if group in document_processors:
                processors = document_processors[group]
                for p in processors:
                    print('Running document processor {}'.format(p))
                    group_documents, resources = p(self.options).process(group_documents, resources)

            completed_documents = []
            chained_documents = []
            for document in group_documents:
                if group is '' or document.group == group:
                    completed_documents.append(document)
                else:
                    chained_documents.append(document)

            if len(chained_documents):
                chained_documents, resources = self.process_documents(chained_documents, resources)
                documents.extend(chained_documents)

            documents.extend(completed_documents)

        return documents, resources

    def process_resources(self, documents, resources):
        # break resources into groups by extension
        resources_by_group = {}
        for resource in resources:
            group = resource.extension
            if group not in resources_by_group:
                resources_by_group[group] = []
            resources_by_group[group].append(resource)

        # process resources by extension
        resources = []
        for group in resources_by_group:
            group_resources = resources_by_group[group]

            if group not in resource_processors:
                group = ''

            if group in resource_processors:
                p = resource_processors[group]
                print('Running resource processor {}'.format(p))
                documents, group_resources = p(self.options).process(documents, group_resources)

            completed_resources = []
            chained_resources = []
            for resource in group_resources:
                if group is '' or resource.extension == group:
                    completed_resources.append(resource)
                else:
                    chained_resources.append(resource)

            if len(chained_resources):
                documents, chained_resources = self.process_resources(documents, chained_resources)
                resources.extend(chained_resources)

            resources.extend(completed_resources)

        return documents, resources

    def process(self, documents, resources):
        # preprocess everything
        for p in preprocessors:
            print('Running pre-processor {}'.format(p))
            documents, resources = p(self.options).process(documents, resources)

        documents, resources = self.process_documents(documents, resources)
        documents, resources = self.process_resources(documents, resources)

        # break documents into groups by extension
        documents_by_group = {}
        for document in documents:
            group = document.extension
            if group not in documents_by_group:
                documents_by_group[group] = []
            documents_by_group[group].append(document)

        # render markup for all documents
        documents = []
        for extension in documents_by_group:
            group_documents = documents_by_group[extension]

            if extension not in markup_processors:
                extension = ''

            if extension in markup_processors:
                processors = markup_processors[extension]
                for p in processors:
                    print('Running markup processor {}'.format(p))
                    group_documents, resources = p(self.options).process(group_documents, resources)

            documents.extend(group_documents)

        # set default document uris
        for document in documents:
            if document.uri is None:
                document.uri = document.path + document.extension

        # postprocess everything
        for p in postprocessors:
            print('Running post-processor {}'.format(p))
            documents, resources = p(self.options).process(documents, resources)

        # finalize document uris
        for document in documents:
            document.uri = urljoin(self.options['site']['root'], document.uri)
            if not document.get('link'):
                document['link'] = document.uri

        return documents, resources

    def write(self, documents, resources):
        render = Render(self.options, documents)

        for document in documents:
            filepath = path.join(self.output_path, document.path)
            filepath += document.extension

            print('Rendering document {}'.format(filepath))
            with open(filepath, 'w') as f:
                f.write(render.render(document))

        for resource in resources:
            filepath = path.join(self.output_path, resource.path)
            filepath += resource.extension

            print('Writing resource {}'.format(filepath))
            with open(filepath, 'wb') as f:
                f.write(resource.content)

    def create_output_hierarchy(self, documents, resources):
        hierarchy = set()

        for document in documents:
            dirname = path.dirname(document.path)
            hierarchy.add(path.join(self.output_path, dirname))

        for resource in resources:
            dirname = path.dirname(resource.path)
            hierarchy.add(path.join(self.output_path, dirname))

        print('Creating output hierarchy: {}'.format(hierarchy))

        for dir in hierarchy:
            os.makedirs(dir, exist_ok=True)

    def run(self):
        documents, resources = self.load()
        documents, resources = self.process(documents, resources)

        self.create_output_hierarchy(documents, resources)
        self.write(documents, resources)



import os.path
from nib import yaml

class Document(dict):
    def __init__(self, path=None, uri=None, group=None, content=None, short=None, **kwargs):
        dict.__init__(self, **kwargs)
        self.path, self.extension = os.path.splitext(path)
        self.uri = uri
        self.group = group
        self.content = content
        self.short = short or content

    def __repr__(self):
        return '<Document({}, {}, {})>'.format(self.path, self.group,
                                               dict.__repr__(self))

    @classmethod
    def from_file(cls, path, options=None):
        metadata, sections = yaml.load(path, supplement=True)
        group = metadata.setdefault('group', None)

        if options:
            path = os.path.relpath(path, options['document_path'])
            defaults = options['defaults']
            for key in defaults:
                metadata.setdefault(key, defaults[key])

        content = sections.pop(0)
        short = content
        if len(sections):
            content += sections.pop(0)

        return Document(path=path,
                        content=content,
                        short=short,
                        **metadata
                        )
